#include "graphics.hpp"

void GraphicsEngine::flip()
{
		SDL_GL_SwapBuffers();
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
}

GLuint GraphicsEngine::loadmodel(const string& mesh,const string& texture)
{
  Model* m = new Model;
  m->load(mesh.c_str(),texture.c_str());
  models.push_back(m);
  return m->dlist;
}

int GraphicsEngine::resizeWindow( int width, int height )
{
    /* Height / width ration */
    GLfloat ratio;

    /* Protect against a divide by zero */
    if ( height == 0 )
		height = 1;

    ratio = ( GLfloat )width / ( GLfloat )height;

    /* Setup our viewport. */
    glViewport( 0, 0, ( GLint )width, ( GLint )height );

    /*
     * change to the projection matrix and set
     * our viewing volume.
     */
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity( );

    /* Set our perspective */
    gluPerspective( 45.0f, ratio, 0.1f, 100.0f );

    /* Make sure we're chaning the model view and not the projection */
    glMatrixMode( GL_MODELVIEW );

    /* Reset The View */
    glLoadIdentity( );

    return( TRUE );
}

GraphicsEngine::GraphicsEngine()
{

  int videoFlags;
  const SDL_VideoInfo *videoInfo;

  if ( SDL_Init( SDL_INIT_VIDEO ) < 0 )
	{
	    std::cout<<"Video initialization failed: " << SDL_GetError( ) << std::endl;
	}

  videoInfo = SDL_GetVideoInfo( );

  if ( !videoInfo )
	{
	    std::cout<<"Video query failed: " << SDL_GetError( ) << std::endl;
	}

    /* the flags to pass to SDL_SetVideoMode */
  videoFlags  = SDL_OPENGL;          /* Enable OpenGL in SDL */
  videoFlags |= SDL_GL_DOUBLEBUFFER; /* Enable double buffering */
  videoFlags |= SDL_HWPALETTE;       /* Store the palette in hardware */
  videoFlags |= SDL_RESIZABLE;       /* Enable window resizing */

  /* This checks to see if surfaces can be stored in memory */
  if ( videoInfo->hw_available )
    videoFlags |= SDL_HWSURFACE;
  else
    videoFlags |= SDL_SWSURFACE;

  /* This checks if hardware blits can be done */
  if ( videoInfo->blit_hw )
    videoFlags |= SDL_HWACCEL;

  /* Sets up OpenGL double buffering */
  SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 2 );

  /* get a SDL surface */
  screen = SDL_SetVideoMode( SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP,
                videoFlags | SDL_FULLSCREEN );

	SDL_ShowCursor(SDL_DISABLE);

  /* Verify there is a surface */
  if ( !screen )
	{
	    std::cout<< "Video mode set failed: " << SDL_GetError( ) << std::endl;
	}

	initGL();

  resizeWindow( SCREEN_WIDTH, SCREEN_HEIGHT );

  fontrenderer = new FontRenderer();

}

void GraphicsEngine::initGL()
{
    /* Enable Texture Mapping ( NEW ) */
    glEnable( GL_TEXTURE_2D );

    /* Enable smooth shading */
    glShadeModel( GL_SMOOTH );

    /* Set the background black */
    glClearColor( 0.0f, 0.0f, 0.0f, 0.5f );

    /* Depth buffer setup */
    glClearDepth( 1.0f );

    /* Enables Depth Testing */
    glEnable( GL_DEPTH_TEST );

    /* The Type Of Depth Test To Do */
    glDepthFunc( GL_LEQUAL );

    /* Really Nice Perspective Calculations */
    glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );

	glMatrixMode(GL_TEXTURE);
	glRotatef(180.0f,0.0f,0.0f,1.0f);
	glScalef(-1.0f,1.0f,1.0f);

	glMatrixMode(GL_MODELVIEW);
}
